! ========================================================================
! Copyright (C) by VMware, Inc. 1991-2011.  All Rights Reserved
!
! $Id: quadbytestring.gs,v 1.14.6.1 2008-04-16 18:43:00 dhenrich Exp $
!
! ========================================================================

! Create QuadByteString class if not already defined (as during conversion),

expectvalue %String
run
| res cls oldCls clsName |
clsName := #QuadByteString.
oldCls := Globals at:clsName otherwise: nil .
oldCls == nil ifTrue:[

  " Must be converting a pre 2.3 DB "
  " Create QuadByteString "

  res := MultiByteString _newKernelSubclass: clsName
    instVarNames: #()
    classVars: #()
    classInstVars: #()
    poolDictionaries: #[]
    inDictionary: Globals
    constraints: #[] 
    instancesInvariant: false
    isModifiable: true
    reservedOop: 991.

  cls := Globals at: clsName.

  "_newKernelSubclass creates oop objects: change format to byte/indexable"
  cls _unsafeAt: 2 put: 5.

  cls immediateInvariant.

] ifFalse: [ res := clsName asString , 'already exists' ].
^ res
%
!  get fresh session to ensure OM root refs to QbString are valid
commit
logout
login

removeallmethods QuadByteString
removeallclassmethods QuadByteString

category: 'For Documentation Installation only'
classmethod: QuadByteString
installDocumentation

| doc txt |
doc := GsClassDocumentation newForClass: self.

txt := (GsDocText new) details:
'A QuadByteString is a string for which each Character occupies four bytes.

 Most of the functionality of this class is inherited from the 
 superclass MultiByteString.  See the documentation there for more details'.
doc documentClassWith: txt.

self description: doc.
%

! ------------------- Class methods for QuadByteString

category 'Formatting'
classmethod: QuadByteString
charSize

"Returns number of bytes that make up a character for this string class."

^ 4
%

category 'Instance Creation'
classmethod: QuadByteString
new: aSize

"(R) Returns a new instance of the receiver, of sufficient size to hold aSize
 Characters of two bytes each."

^ super new: (aSize * (self charSize))
%

category 'Instance Creation'
classmethod: QuadByteString
new

"(R) Returns a new instance of the receiver."

^ self basicNew
%

category 'Instance Creation'
classmethod: QuadByteString
new: aSize

"(R) Returns a new instance of the receiver, of sufficient size to hold aSize
 Characters of four bytes each."

^ self basicNew: (aSize * (self charSize))
%

! ------------------- Instance methods for QuadByteString
category 'Formatting'
method: QuadByteString
charSize

"Returns number of bytes that make up a character for this string class."

^ 4
%

category: 'Searching'
method: QuadByteString
speciesForCollect

"(R) Returns a class, an instance of which should be used as the result
 of collect: or other projections applied to the receiver."

^ QuadByteString
%

category: 'Other Comparisons'
method: QuadByteString
collatingTable

"Return the appropriate collating table for this class"

^ QuadByteAsciiCollatingTable
%

category: 'New Indexing Comparison'
method: QuadByteString
_idxForCompareEqualTo: arg

""

^arg _idxForCompareEqualToQuadByteString: self
%

category: 'Accessing'
method: QuadByteString
at: anIndex

"Returns the Character at anIndex."

<primitive: 672>
(anIndex _isInteger)
  ifTrue: [^ self _errorIndexOutOfRange: anIndex]
  ifFalse: [^ self _errorNonIntegerIndex: anIndex].
self _primitiveFailed: #at: .
self _uncontinuableError
%

